/* Stats - IRC Statistical Services for Hybrid, csircd, and IRCnet 2.10
** proto.h - Function declarations
**
** Copyright W. Campbell and others.  See README for more details
** Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
**                      IRCnet IRCD developers.
**
** $Id: proto.h,v 1.110 2002/07/13 19:19:12 wcampbel Exp $
*/

#ifndef PROTO_H
#define PROTO_H

#include "stats.h"

/* Macros that were in m_stats.h */
#define DecreaseOpers(x)        { (x)->opers--; netinfo.opers--; }
#define DecreaseUsers(x)        { (x)->users--; netinfo.users--; }
#define DecreaseServers()       netinfo.servers--;

/* Macros to replace ugly inlines */
#define msgtoken(x,y)		generictoken(' ', (x), (y))
#define tabtoken(x,y)		generictoken('\t', (x), (y))

/* Fixes for broken operating systems */
#ifndef HAVE_STRLCAT
size_t strlcat(char *, const char *, size_t);
#endif

#ifndef HAVE_STRLCPY
size_t strlcpy(char *, const char *, size_t);
#endif

/* Simplification macros */
#define IsPrivmsg()	(settings.flags & FLAG_PRIVMSG)
#define IsLag()		(settings.flags & FLAG_LAG)
#define IsOnlyOpers()	(settings.flags & FLAG_ONLYOPERS)
#define IsAOnlyOpers()  (settings.flags & FLAG_AONLYOPERS)
#define IsHtml()	(settings.flags & FLAG_HTML)
#define IsDump()	(settings.flags & FLAG_DUMP)
#define IsSetOper()	(settings.flags & FLAG_SETOPER)
#define IsFlood()	(settings.flags & FLAG_FLOOD)
#define IsVHost()	(settings.flags & FLAG_VHOST)
#define IsSilent()	(settings.flags & FLAG_SILENT)
#define IsNotifyEOB()	(settings.flags & FLAG_NOTIFYEOB)
#define IsOper(u)	((u)->flags & UFLAG_ISOPER)
#define IsAway(u)	((u)->flags & UFLAG_ISAWAY)
#define IsFloodSent(u)	((u)->flags & UFLAG_FLOODSENT)
#define IsOperOn(myu)	((myu)->flags & MFLAG_ISON)
#define IsDebug()	(me.cmdflags & CMD_DEBUG)
#define IsNoFork()	(me.cmdflags & CMD_NOFORK)
#define IsSHide(s)	((s)->flags & SFLAG_HIDE)

#define IsSPPrivmsg()	(sp.flags & SPF_PRIVMSG)
#define IsSPChanmsg()	(sp.flags & SPF_CHANMSG)
#define IsSPEnableText()	(sp.flags & SPF_ENABLE_TEXT)
#define IsSPEnableExpire()	(sp.flags & SPF_EXPIRE)
#define IsSPReport()	(sp.flags & SPF_REPORT)
#define IsSPHide()	(sp.flags & SPF_HIDE)

#define SetEOB(s)	((s)->flags |= SFLAG_EOB)
#define IsSetEOB(s)	((s)->flags & SFLAG_EOB)

/* External variables... */
extern int servsock;

#ifdef SPAMSERV
extern char s_Spam[SPAMLEN];
#endif

extern const char version[];

#ifdef SUPPORT_MSG_UH
#ifdef STATSERV
extern char s_UH[MAXUSER + MAXHOST + 2];
#endif
#ifdef SPLITSERV
extern char sp_UH[MAXUSER + MAXHOST + 2];
#endif
#ifdef JUPESERV
extern char j_UH[MAXUSER + MAXHOST + 2];
#endif
extern char a_UH[MAXUSER + MAXHOST + 2];
#endif

#ifdef SUPPORT_MSG_OPERS
extern char s_OPERS[MAXHOST + 7];
#endif

#if defined(SPLITSERV) || defined(SPYCHAN)
int is_delayed;
time_t notice;
#endif

#ifdef SPLITSERV
struct SpServer* map_head;
struct SpServer* map_uplink;
dlink_list split_table;
dlink_list server_table;

/* split-util.c */
void dump_map();
void html_map(FILE *, struct SpServer*, int, int *);
struct mapreturn PrintMap(char*, struct SpServer*, int, int *);
void PrintSplit(char*);
void PrintDump(char *);
void Forget(char * origin, char * serv);
void Add(char * origin, char * serv);
void Update(char * origin);
void expire_splits(void);

/* split-data.c */
void SpInitTables();
struct SpServer* SpNewServer(char *, char *);
struct SpServer* SpFindServer(char *);
void SpDelServer(char *);
void SpNewSplit(char *);
void SpDelSplit(char *);
struct SpServer* SpFindSplit(char *);

/* splitserv.c */
void splitserv(char *, char *, char *);

#endif

/* adminserv.c */
void adminserv(char *, char *);

/* sock.c */
int ConnectTo(char *, int);
void read_loop();
void sts(char *, ...);

/* util.c */
char *timediff(time_t);
void log(char *, ...);
#ifdef SPLITSERV
void splog(char *, ...);
#endif
void ResetLogs();
char *sctime(time_t);
char *sftime(time_t);
char *irc_time(time_t);
#if defined(CRYPT) || defined(MD5)
char *make_salt();
#endif /* CRYPT || MD5 */
int tokenize(char *, char **);
int generictoken(char, char *, char **);
void *smalloc(size_t);
unsigned long HASH(const unsigned char *, int);
char *strlower(char *);
char *VersionFlags();
#ifdef OPER_MODES
char opermode(struct User *);
#endif
void SigTerm(int);
void SigUsr1(int);
void SigHup(int);
int is_channel(char *);
void alert(char *, ...);
void spy(char *, ...);
void debug(char *, ...);
void InitLists(void);
void ExpandParv(char *, int, int, int, char **);

/* conf.c */
void InitDefaults();
void VerifyConf();
void ConfLoad();
void rehash();

/* init.c */
void login();
#ifdef STATSERV
void init_statserv();
#endif
#ifdef SPAMSERV
void init_spam();
#endif
#ifdef STATSERV
void init_join();
#endif
void init_admin();
#ifdef SPLITSERV
void init_splitserv();
void init_spjoin();
#endif
#ifdef JUPESERV
void init_jupeserv();
#endif
#ifdef SPYMODE
void init_spy_join();
#endif
#ifdef ALERTMODE
void init_alert_join();
#endif
void InitMessageStrings();

/* ircd.c */
void parse();
void privmsg(char *, const char *, int, char *, ...);
void send_help_file(char *, char *, int, char *, char *);
void wallops(char *, char *, ...);
int flood(struct User *);
void SendQuit(struct User *, int, char *);

/* timer.c */
int need_rehash;
void chk();
void TimerReset();

/* statserv.c */
#ifdef STATSERV
#ifdef IRCD_2_10
void statserv(char *, char *, char *, int);
#else
void statserv(char *, char *, char *);
#endif
#endif

/* spam.c */
#ifdef SPAMSERV
void spam(char *, char *);
#endif

/* users.c */
struct Server *serverlist[S_TABLE_SIZE];
struct User *userlist[U_TABLE_SIZE];
struct MyUser *myuhead;
#ifdef IRCD_2_10
dlink_list servicehead;
#endif
#ifdef IRCD_2_10
void AddUser(char *, char *, char *, struct Server *);
#else
void AddUser(char *, char *, char *, char *);
#endif
void DelUser(char *);
void Change_User(struct User *, char *);
struct User *finduser(char *);
void UserMode(char *, char *);
void init_user_hash();
#ifdef IRCD_2_10
void AddServer(char *, int, int, char *, char *);
struct Server *findservtoken(int);
#else
void AddServer(char *, int, char *, char *);
#endif
void FlushServer(char *);
void DelServer(char *);
struct Server *findserver(char *);
void init_server_hash();
void LoadMyUsers();
void SaveMyUsers();
void DeleteMyUser(char *);
struct MyUser *findmyuser(char *);
#ifdef IRCD_2_10
struct Service *findservice(char *, char *);
void AddService(char *, char *, char *, char *);
void DelService(char *, char *);
void DelServicesByServer(char *);
#endif

/* tld.c */
struct TLD *tldlist[T_TABLE_SIZE];
struct TLD *findtld(char *);
struct TLD *AddTLD(struct User *);
void LoadTLD();
void init_tld();

/* stats.c */
struct Stats *shead;
dlink_list hiddenhead;
void IncreaseOpers(struct Server *);
void IncreaseUsers(struct Server *);
void IncreaseServers();
void AddStats(struct Server *);
struct Stats *findstats(char *);
void SaveStats();
void LoadStats();
void AddHidden(char *, int);
#if 0
void DelHidden(char *);
#endif
struct Hidden* findhidden(char *);

/* html.c */
void DumpHTML();

/* datadump.c */
void datadump();

#ifdef JUPESERV

dlink_list jupehead;

/* jupe.c */
void LoadJupes();
void SaveJupes();
void AddJupe(char *, char *, long, char *);
void InitJupe(struct Jupe *);
void DelJupe(char *);
struct Jupe* FindJupe(char *);
void FlushJupeList();

/* jupeserv.c */
void jupeserv(char *, char *);

#endif

/* info.c - autogenerated */
char *stats_version;
char *stats_date;

#endif
