/* Stats - IRC Statistical Services for Hybrid, csircd, and IRCnet 2.10
** sock.c - TCP/IP socket management
**
** Copyright W. Campbell and others.  See README for more details
** Some code Copyright: Jonathan George, Kai Seidler, ircd-hybrid Team,
**                      IRCnet IRCD developers.
**
** $Id: sock.c,v 1.42 2002/07/28 02:31:18 wcampbel Exp $
*/

#include "stats.h"
#include "struct.h"
#include "proto.h"
#include "setup.h"

fd_set readfds, nullfds;

static int Server_receiveFrom(int, char *);
static void Service_processPacket(char *);

#ifdef HAVE_GETADDRINFO
static struct addrinfo * gethostinfo(char const *host, int port);

int ConnectTo(char *host, int port)
{
	struct addrinfo *hostres, *res;
	struct addrinfo *bindres = NULL;
	int s = -1;

	hostres = gethostinfo(host, port);
	if (hostres == NULL)
	{
		log("FATAL:  Lookup error in CONNECT_TO, exiting");
		exit(6);
	}
	if (IsVHost())
	{
		bindres = gethostinfo(settings.vhost, 0);
		if (bindres == NULL)
		{
			log("FATAL:  Lookup error in VHOST, exiting");
			exit(6);
		}
	}
	for (res = hostres; res; res = res->ai_next)
	{
		s = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
		if (s < 0)
			continue;
		if (IsVHost())
		{
			if (bind(s, bindres->ai_addr, bindres->ai_addrlen) < 0)
			{
				log("Cannot bind to VHost");
				return (-1);
			}
		}
		if (connect(s, res->ai_addr, res->ai_addrlen) == 0)
			break;
		log("%s", strerror(errno));
		close(s);
	}
	freeaddrinfo(hostres);
	if (res == NULL)
	{
		log("Cannot connect to host");
		return (-1);
	}

	return s;
}
#else
/* Initiate the connection to the server, returns the FD */
int ConnectTo(char *host, int port)
{
	struct hostent *hp;
	struct sockaddr_in sa;
	struct in_addr * in;
	int s;

	if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	{
		log("Cannot create socket()");
		return (-1);
	}

        if (IsVHost())
	{
                memset((void *) &sa, '\0', sizeof(sa));
                sa.sin_family = AF_INET;
                if ((hp = gethostbyname (settings.vhost)) == NULL)
		{
			log("Unable to gethostbyname(%s)", settings.vhost);
                        return (-1);
                }
                in = (struct in_addr *) (hp->h_addr_list[0]);
                sa.sin_addr.s_addr = in->s_addr;
                sa.sin_port = 0;
                if (bind(s, (struct sockaddr *) &sa, sizeof(sa)) < 0)
		{
                        close(s);
			log("Unable to bind to VHOST %s", settings.vhost);
                        return (-1);
                }
        }

        if ((hp = gethostbyname (host)) == NULL)
        {
                log("Cannot gethostbyname(%s)", host);
                return (-1);
        }

	memset((void *) &sa, '\0', sizeof(sa));
	sa.sin_family = AF_INET;
	sa.sin_port = htons (port);
	memcpy((void *) &sa.sin_addr, (void *) hp->h_addr, hp->h_length);

	if (connect (s, (struct sockaddr *) &sa, sizeof (sa)) < 0)
	{
		close (s);
		log("Cannot connect()");
		return (-1);
	}

	debug("Connected to server(%s) port(%d)", host, port);
	return s;
}
#endif

/* Here is where it all happens, get a line from the server
** and send it off to the parse function.  Also this is where
** chk, the timer function, is called.  
*/
void read_loop()
{
	int SelectResult;
	struct timeval TimeOut;
	static char buf[BUFSIZE + 1];

	while (TRUE)
	{
		memset(buf, '\0', BUFSIZE + 1);
		chk();
		FD_ZERO(&readfds);
		/* To improve the timing loop a little, I am going
		** to reduce the seconds variable from 500 down to
		** 10 for now.
		** NOTE:  Changed down to 1, it appears to be fine
		*/
		TimeOut.tv_sec = 1;
		TimeOut.tv_usec = 0L;
		FD_SET(servsock, &readfds);
		if ((SelectResult = select(FD_SETSIZE, &readfds, &nullfds,
			&nullfds, &TimeOut)) > 0)
		{
			if (!Server_receiveFrom(servsock, buf))
			{
				log("Lost connection to server.");
				return;
			}
			Service_processPacket(buf);
		}
		else
		{
			/* select returns 0 when the TimeOut is hit */
			if (SelectResult == 0)
			{
				/* simply a timeout, needed for chk() */
			}
			else if ((SelectResult == -1) && (errno == EINTR))
			{
				/* Interrupted system call, we got a signal */
			}
			else
			{
				/* select returned -1, an error condition of
				** some kind (we don't care what) occurred.
				*/
				log("Lost connection to server.");
				return;
			}
		}
	}
}

/* send a line of text to the server, Append the \n */
void sts(char *fmt,...)
{
	va_list ap;
	char buf[BUFSIZE];

	va_start (ap, fmt);
	vsnprintf (buf, BUFSIZE, fmt, ap);

	debug("SENT: %s", buf);
	strlcat(buf, "\n", BUFSIZE);

	netinfo.outb += strlen(buf);

	if ((write(servsock, buf, strlen(buf))) == -1)
	{
		log("Write error.");
		close(servsock);
		/* hmm, is close the right thing to do here?
		** I'm sure it's still better than exit(0)
		*/

		/* exit(0); */
	}

	va_end (ap);
}

/* These two functions were lifted from oswald's NoteServ */
static int Server_receiveFrom(int fd, char *buffer)
{
    /* This mustn't be BUFSIZE!!! */
    /* unless buffer is BUFSIZE + 1 */
    int n = read(fd, buffer, BUFSIZE);
    buffer[n] = '\0';
    return n;
}

static void Service_processPacket(char *buffer)
{
    char *ptr;
    char buffer2[BUFSIZE * 2];
    static char tmp[BUFSIZE * 2];

    while ((ptr = strchr(buffer, '\n')))
    {
        *ptr = '\0';
        if (*(ptr - 1) == '\r')
            *(ptr - 1) = '\0';
        snprintf(buffer2, (BUFSIZE * 2), "%s%s", tmp, buffer);
        *tmp = '\0';
        parse(buffer2);
        buffer = ptr + 1;
    }
    if (*buffer)
    {
        strlcpy(tmp, buffer, (BUFSIZE * 2) - strlen(tmp));
    }
}

#ifdef HAVE_GETADDRINFO
/* Stolen from FreeBSD's whois client, modified for Stats by W. Campbell */
static struct addrinfo * gethostinfo(char const *host, int port)
{
        struct addrinfo hints, *res;
        int error;
	char portbuf[6];

        memset(&hints, 0, sizeof(hints));
        hints.ai_flags = 0;
        hints.ai_family = AF_UNSPEC;
        hints.ai_socktype = SOCK_STREAM;
	snprintf(portbuf, 6, "%d", port);
        error = getaddrinfo(host, portbuf, &hints, &res);
        if (error) {
                log("gethostinfo error: %s: %s", host, gai_strerror(error));
                return (NULL);
        }
        return (res);
}
#endif
